#ifndef irdeto2_xx
#define irdeto2_xx


#include <stdlib.h>
#include <string.h>
#include "ssl/helper.h"
#include "emu.h"
#include <openssl/des.h>

class cIrdeto2 {
private:
  DES_key_schedule ks1, ks2;
  void ScheduleKey(const unsigned char *key);
  void DES3(unsigned char *data, int mode);

  void PrepareSeed(unsigned char *seed, const unsigned char *key);
  void NanoDecrypt(unsigned char *data, int i, int len, const unsigned char *key, const unsigned char *iv);
protected:
  void Encrypt(unsigned char *data, const unsigned char *seed, const unsigned char *key, int len);
  void Decrypt(unsigned char *data, const unsigned char *seed, const unsigned char *key, int len);
  bool CalculateHash(const unsigned char *key, const unsigned char *iv, const unsigned char *data, int len);
public:
  void HEXDUMP(char *line,unsigned char *buffer, int length);
  bool ProcessECM(EmuInfo *ecm, unsigned char *data, char *keyfile);
  unsigned char cw[16];
  };

void cIrdeto2::HEXDUMP(char *line,unsigned char *buffer, int length)
{
	int i;
        printf("%s",line);
	for (i = 0;i < length;i++)
	{
		if (i % 16 == 0) printf("\n%04X=", i);
		printf("%02X", buffer[i]);
	}
	printf("\n");
}

void cIrdeto2::ScheduleKey(const unsigned char *key)
{
  DES_key_sched((DES_cblock *)key,&ks1);
  DES_key_sched((DES_cblock *)(key+8),&ks2);
}

void cIrdeto2::DES3(unsigned char *data, int mode)
{
  int m1, m2;
  if(mode) { m1=DES_DECRYPT; m2=DES_ENCRYPT; }
  else     { m1=DES_ENCRYPT; m2=DES_DECRYPT; }
  DES_ecb_encrypt((DES_cblock *)data,(DES_cblock *)data,&ks1,m1);
  DES_ecb_encrypt((DES_cblock *)data,(DES_cblock *)data,&ks2,m2);
  DES_ecb_encrypt((DES_cblock *)data,(DES_cblock *)data,&ks1,m1);
}

void cIrdeto2::Encrypt(unsigned char *data, const unsigned char *seed, const unsigned char *key, int len)
{
  ScheduleKey(key);
  len&=~7;
  const unsigned char *tmp=seed;
  for(int i=0; i<len; i+=8) {
    xxor(&data[i],8,&data[i],tmp); tmp=&data[i];
    DES3(&data[i],0);
    }
}

void cIrdeto2::Decrypt(unsigned char *data, const unsigned char *seed, const unsigned char *key, int len)
{
  ScheduleKey(key);
  len&=~7;
  unsigned char buf[2][8];
  int n=0;
  memcpy(buf[n],seed,8);
  for(int i=0; i<len; i+=8,data+=8,n^=1) {
    memcpy(buf[1-n],data,8);
    DES3(data,1);
    xxor(data,8,data,buf[n]);
    }
}

bool cIrdeto2::CalculateHash(const unsigned char *key, const unsigned char *iv, const unsigned char *data, int len)
{
  ScheduleKey(key);
  unsigned char cbuff[8];
  memset(cbuff,0,sizeof(cbuff));
  len-=8;
  for(int y=0; y<len; y+=8) {
    if(y<len-8) {
      xxor(cbuff,8,cbuff,&data[y]);
      //LDUMP(L_SYS_VERBOSE,cbuff,8,"3DES XOR in:");
      }
    else {
      int l=len-y;
      xxor(cbuff,l,cbuff,&data[y]);
      xxor(cbuff+l,8-l,cbuff+l,iv+8);
      //LDUMP(L_SYS_VERBOSE,cbuff,8,"3DES XOR(%d) in:",8-l);
      }
    DES3(cbuff,0);
    //LDUMP(L_SYS_VERBOSE,cbuff,8,"3DES out:");
    }
  HEXDUMP("CryptBuffer:",cbuff,8);

  return memcmp(cbuff,&data[len],8)==0;
}

// -- cSystemIrd2 --------------------------------------------------------------

#define NANOLEN(_a) ((_a) ? ((_a)&0x3F)+2 : 1)


void cIrdeto2::PrepareSeed(unsigned char *seed, const unsigned char *key)
{
  unsigned char blank[16];
  memset(blank,0,16);
  Encrypt(seed,blank,key,16);
}

void cIrdeto2::NanoDecrypt(unsigned char *data, int i, int len, const unsigned char *key, const unsigned char *iv)
{
  while(i<len) {
    int l=NANOLEN(data[i+1]);
    printf("NANODEC -> data(i) %02x  len:%d\n",data[i],l);
    switch(data[i]) {
      case 0x10:
      case 0x50: if(l==0x13 && i<=len-l) Decrypt(&data[i+3],iv,key,16); printf("DEC 10-50\n"); break;
	  case 0x77:
      case 0x78: 
			if(l==0x14 && i<=len-l) Decrypt(&data[i+4],iv,key,16);
			//data[i+4]=0x24; 
			//data[i+5]=0xD4;
			//data[i+6]=0xf4;
			//data[i+7]=0xfe;
			//data[i+8]=0xf7;
			//data[i+9]=0x9c;
			//data[i+10]=0x32;
			//data[i+11]=0x21;
			//data[i+3]=0x78;
			printf("DEC 77-78\n"); break;
			
	  //case 0xd6: Decrypt(&data[i],iv,key,16);	printf("DEC d8\n"); break;
      }
    i+=l;
    }
}

bool cIrdeto2::ProcessECM(EmuInfo *ecm, unsigned char *data, char *keyfile)
{
// M1=ecm_seed   M2=ecm_IV 
//I 060300 M2 87A3DBFC22B199E762C649E08F2A5E65 ; Al Jazeera Sport ricky 2013-01-06 13:40:59 -----IV
//I 060300 M1 903C3379A8698CE1D829CE82010C2F31 ; Al Jazeera Sport coscos1222 2012-12-04 21:41:11-----seed
//I 060300 06 7B4412B80604D053DE5A4A057E161A5B ; Al Jazeera Sport tonytr 2013-01-13 18:14:35 
  unsigned char ECM_IV[16];
  unsigned char ECM_Seed[16];
  unsigned char key[16];
  	
  //char hcaid[8]; 	sprintf(hcaid,"%04X%02X",ecm->caId,data[8]);
  //char hkeynr[8];	sprintf(hkeynr,"%02X",data[9]); 
  
  //printf (hcaid,hkeynr);
  
  if (!FindKey("I","060000","EM",ecm,keyfile)) return false;		// ECM
  keystr_to_key(ecm->indata,data);
  
  if (!FindKey("I","060000","M1",ecm,keyfile)) return false;		// Seed
  keystr_to_key(ecm->indata,ECM_Seed);

  if (!FindKey("I","060000","M2",ecm,keyfile)) return false;		// IV 
  keystr_to_key(ecm->indata,ECM_IV);

  if (!FindKey("I","060000","06",ecm,keyfile)) return false;		// Op Key
  keystr_to_key(ecm->indata,key);
  
  HEXDUMP("Original IV   :",ECM_IV,16);
  HEXDUMP("Original Seed :",ECM_Seed,16);
  HEXDUMP("Original Key  :",key,16);
	
  int len=data[11];

  if( !(SCT_LEN(data)==len+12) ) 
	{
	printf("Bad ECM length\n");
    return false;
    }
  HEXDUMP("Original ECM :",data,len);
  
  PrepareSeed(ECM_Seed,key);

  data+=12; 
  Decrypt(data,ECM_IV,ECM_Seed,len);
  HEXDUMP("CRYPTED ECM :",data,len);
  int i=(data[0]&7)+1;
  if (data[i-1]==0x77 && data[i]==0x12) i--;
  //printf("I : %d\n",i); // bu 5 hata 4 olack   ,,  +1 silindi
  NanoDecrypt(data,i,len-8,key,ECM_IV);
  HEXDUMP("UNCRYPTED ECM :",data,len);
  /*
  int z=-6;
  while (z<11)
		{
		if (CalculateHash(ECM_Seed,ECM_IV,data-z,len+z))
			{
				printf("Hash OK: %d",z);
				break;
			}
		z=z+1;
		}
	*/	
  if (CalculateHash(ECM_Seed,ECM_IV,data-6,len+6)) {
    HEXDUMP("Irdeto2 RAWECM",data-6,len+6);
    while(i<len-8) {
      int l=NANOLEN(data[i+1]);
      switch(data[i]) {
        case 0x78:
		case 0x77:
          memcpy(cw,&data[i+4],16);
		  HEXDUMP("Irdeto2 DCW :",cw,16);
          //ks.OK(pk);
          return true;
        }
      i+=l;
      }
    }
  else {
	printf("hash failed\n");
    }
  return false;
}

#endif
